/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Taxonomy.java,v 584a8002bbf7 2010/07/17 13:10:42 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy;

import it.tidalwave.util.As;
import it.tidalwave.util.Displayable;
import javax.annotation.Nonnull;
import java.io.Serializable;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * This class models a hyerarchical structure of {@link TaxonomyConcept}s. Every {@link Taxonomy} has a single
 * "top concept"; each concept has a name, an identifier and zero ore more child concepts.
 *
 * @author  Fabrizio Giudici
 * @version $Id: Taxonomy.java,v 584a8002bbf7 2010/07/17 13:10:42 fabrizio $
 *
 **********************************************************************************************************************/
public interface Taxonomy extends Lookup.Provider, Displayable, Serializable, As
  {
    /*******************************************************************************************************************
     *
     * Creates a top concept for this {@code Taxonomy}.
     * 
     * @param  displayName   the display name of the concept
     * @param  id            the id
     *
     ******************************************************************************************************************/
    @Nonnull
    public TaxonomyConcept createTopConcept (@Nonnull String displayName,
                                             @Nonnull String id);

    /*******************************************************************************************************************
     *
     * TBD
     * 
     * @param  displayName   the display name of the concept
     * @param  id            the id
     *
     ******************************************************************************************************************/
    @Nonnull
    public TaxonomyConcept createConcept (@Nonnull String displayName,
                                          @Nonnull String id);

    /*******************************************************************************************************************
     *
     * Returns the top concept of this {@code Taxonomy}.
     *
     * @return  the top concept
     *
     ******************************************************************************************************************/
    @Nonnull
    public TaxonomyConcept getTopConcept();

    /*******************************************************************************************************************
     *
     * Returns a {@link Finder} for performing queries.
     *
     * @return  the {@code Finder}
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<TaxonomyConcept> findConcepts();

    /*******************************************************************************************************************
     *
     * Finds or creates an {@link AnonymousTaxon} with a given id.
     *
     * @param   id  the id
     * @return  the {@code AnonymousTaxon}
     *
     ******************************************************************************************************************/
    @Nonnull
    public AnonymousTaxon findOrCreateAnonymousTaxon (@Nonnull String id);
  }
